# Generated by Django 4.1.10 on 2023-09-19 17:12

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0016_wmslayer_date_format'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='more_info',
            field=wagtail.fields.StreamField([('more_info', wagtail.blocks.StructBlock([('link_text', wagtail.blocks.CharBlock(label='Link text', required=True)), ('link_url', wagtail.blocks.URLBlock(label='Link Url', required=True)), ('text', wagtail.blocks.TextBlock(label='Short description text', required=False)), ('is_button', wagtail.blocks.BooleanBlock(default=True, label='Format link as action button', required=False)), ('show_arrow', wagtail.blocks.BooleanBlock(default=True, label='Show arrow in button', required=False))], label='Info link'))], blank=True, null=True, use_json_field=True, verbose_name='More Info'),
        ),
        migrations.AlterField(
            model_name='wmslayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-mm-dd HH:MM', 'Hourly - (E.g 2023-01-01 00:00)'), ('yyyy-mm-dd', 'Daily - (E.g 2023-01-01)'), ('yyyy-mm', 'Monthly - (E.g 2023-01)'), ('mmmm yyyy', 'Monthly - (E.g January 2023)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)')], max_length=100, null=True),
        ),
    ]
