# Generated by Django 4.1.10 on 2023-07-12 08:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0009_stationsettings_popup_props'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='stationsettings',
            name='popup_props',
        ),
    ]
