{% extends "admin/change_form.html" %}

{% block after_field_sets %}

    <script>
        var baseEndpoint = 'api/file-raster';
    </script>

    {% include 'admin/django_large_image/_include/geojs.html' %}

{% endblock %}