.go-to-datasets {
    margin-bottom: 40px;
}


#preview-map {
    margin-top: 30px;
    height: 500px;
    width: 100%;
}

select, option {
    cursor: pointer
}


.layer-timestamps-wrapper {
    position: absolute;
    z-index: 1;
    top: 12px;
    left: 12px;
}

.colorscale-control-wrapper {
    position: absolute;
    z-index: 1;
    top: 12px;
    right: 12px;
}

