# Generated by Django 4.1.10 on 2023-07-10 19:32

from django.db import migrations, models
import geomanager.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0006_stationsettings'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='stationsettings',
            name='id_field',
        ),
        migrations.AddField(
            model_name='stationsettings',
            name='bounds',
            field=geomanager.fields.ListField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name='stationsettings',
            name='geom_type',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
